@@ -11,6 +11,8 @@ from cognite.client.data_classes import (
     FileMetadataFilter,
     FileMetadataList,
     FileMetadataUpdate,
+    Label,
+    LabelFilter,
     TimestampRange,
 )
 
@@ -33,6 +35,7 @@ class FilesAPI(APIClient):
         asset_subtree_external_ids: List[str] = None,
         data_set_ids: List[int] = None,
         data_set_external_ids: List[str] = None,
+        labels: LabelFilter = None,
         source: str = None,
         created_time: Union[Dict[str, Any], TimestampRange] = None,
         last_updated_time: Union[Dict[str, Any], TimestampRange] = None,
@@ -40,6 +43,7 @@ class FilesAPI(APIClient):
         source_modified_time: Union[Dict[str, Any], TimestampRange] = None,
         uploaded_time: Union[Dict[str, Any], TimestampRange] = None,
         external_id_prefix: str = None,
+        directory_prefix: str = None,
         uploaded: bool = None,
         limit: int = None,
     ) -> Generator[Union[FileMetadata, FileMetadataList], None, None]:
@@ -60,6 +64,7 @@ class FilesAPI(APIClient):
             asset_subtree_external_ids (List[str]): List of asset subtrees external ids to filter on.
             data_set_ids (List[int]): Return only files in the specified data sets with these ids.
             data_set_external_ids (List[str]): Return only files in the specified data sets with these external ids.
+            labels (LabelFilter): Return only the files matching the specified label(s).
             source (str): The source of this event.
             source_created_time (Union[Dict[str, Any], TimestampRange]): Filter for files where the sourceCreatedTime field has been set and is within the specified range.
             source_modified_time (Union[Dict[str, Any], TimestampRange]): Filter for files where the sourceModifiedTime field has been set and is within the specified range.
@@ -67,6 +72,7 @@ class FilesAPI(APIClient):
             last_updated_time (Union[Dict[str, int], TimestampRange]):  Range between two timestamps. Possible keys are `min` and `max`, with values given as time stamps in ms.
             uploaded_time (Union[Dict[str, Any], TimestampRange]): Range between two timestamps
             external_id_prefix (str): External Id provided by client. Should be unique within the project.
+            directory_prefix (str): Filter by this (case-sensitive) prefix for the directory provided by the client.
             uploaded (bool): Whether or not the actual file is uploaded. This field is returned only by the API, it has no effect in a post body.
             limit (int, optional): Maximum number of files to return. Defaults to return all items.
 
@@ -88,6 +94,7 @@ class FilesAPI(APIClient):
             asset_external_ids=asset_external_ids,
             root_asset_ids=root_asset_ids,
             asset_subtree_ids=asset_subtree_ids,
+            labels=labels,
             source=source,
             created_time=created_time,
             last_updated_time=last_updated_time,
@@ -95,6 +102,7 @@ class FilesAPI(APIClient):
             source_created_time=source_created_time,
             source_modified_time=source_modified_time,
             external_id_prefix=external_id_prefix,
+            directory_prefix=directory_prefix,
             uploaded=uploaded,
             data_set_ids=data_set_ids,
         ).dump(camel_case=True)
@@ -215,6 +223,7 @@ class FilesAPI(APIClient):
         asset_subtree_external_ids: List[str] = None,
         data_set_ids: List[int] = None,
         data_set_external_ids: List[str] = None,
+        labels: LabelFilter = None,
         source: str = None,
         created_time: Union[Dict[str, Any], TimestampRange] = None,
         last_updated_time: Union[Dict[str, Any], TimestampRange] = None,
@@ -222,6 +231,7 @@ class FilesAPI(APIClient):
         source_modified_time: Union[Dict[str, Any], TimestampRange] = None,
         uploaded_time: Union[Dict[str, Any], TimestampRange] = None,
         external_id_prefix: str = None,
+        directory_prefix: str = None,
         uploaded: bool = None,
         limit: int = 25,
     ) -> FileMetadataList:
@@ -239,6 +249,7 @@ class FilesAPI(APIClient):
             asset_subtree_external_ids (List[str]): List of asset subtrees external ids to filter on.
             data_set_ids (List[int]): Return only files in the specified data sets with these ids.
             data_set_external_ids (List[str]): Return only files in the specified data sets with these external ids.
+            labels (LabelFilter): Return only the files matching the specified label filter(s).
             source (str): The source of this event.
             created_time (Union[Dict[str, int], TimestampRange]):  Range between two timestamps. Possible keys are `min` and `max`, with values given as time stamps in ms.
             last_updated_time (Union[Dict[str, int], TimestampRange]):  Range between two timestamps. Possible keys are `min` and `max`, with values given as time stamps in ms.
@@ -246,6 +257,7 @@ class FilesAPI(APIClient):
             source_created_time (Union[Dict[str, Any], TimestampRange]): Filter for files where the sourceCreatedTime field has been set and is within the specified range.
             source_modified_time (Union[Dict[str, Any], TimestampRange]): Filter for files where the sourceModifiedTime field has been set and is within the specified range.
             external_id_prefix (str): External Id provided by client. Should be unique within the project.
+            directory_prefix (str): Filter by this (case-sensitive) prefix for the directory provided by the client.
             uploaded (bool): Whether or not the actual file is uploaded. This field is returned only by the API, it has no effect in a post body.
             limit (int, optional): Max number of files to return. Defaults to 25. Set to -1, float("inf") or None
                 to return all items.
@@ -274,6 +286,14 @@ class FilesAPI(APIClient):
                 >>> c = CogniteClient()
                 >>> for file_list in c.files(chunk_size=2500):
                 ...     file_list # do something with the files
+
+            Filter files based on labels::
+
+                >>> from cognite.client import CogniteClient
+                >>> from cognite.client.data_classes import LabelFilter
+                >>> c = CogniteClient()
+                >>> my_label_filter = LabelFilter(contains_all=["WELL LOG", "VERIFIED"])
+                >>> file_list = c.files.list(labels=my_label_filter)
         """
         if root_asset_ids or root_asset_external_ids:
             root_asset_ids = self._process_ids(root_asset_ids, root_asset_external_ids, wrap_ids=True)
@@ -290,6 +310,7 @@ class FilesAPI(APIClient):
             asset_external_ids=asset_external_ids,
             root_asset_ids=root_asset_ids,
             asset_subtree_ids=asset_subtree_ids,
+            labels=labels,
             source=source,
             created_time=created_time,
             last_updated_time=last_updated_time,
@@ -297,6 +318,7 @@ class FilesAPI(APIClient):
             source_created_time=source_created_time,
             source_modified_time=source_modified_time,
             external_id_prefix=external_id_prefix,
+            directory_prefix=directory_prefix,
             uploaded=uploaded,
             data_set_ids=data_set_ids,
         ).dump(camel_case=True)
@@ -347,6 +369,7 @@ class FilesAPI(APIClient):
         self, item: Union[FileMetadata, FileMetadataUpdate, List[Union[FileMetadata, FileMetadataUpdate]]]
     ) -> Union[FileMetadata, FileMetadataList]:
         """`Update files <https://docs.cognite.com/api/v1/#operation/updateFiles>`_
+        Currently, a full replacement of labels on a file is not supported (only partial add/remove updates). See the example below on how to perform partial labels update.
 
         Args:
             item (Union[FileMetadata, FileMetadataUpdate, List[Union[FileMetadata, FileMetadataUpdate]]]): file(s) to update.
@@ -371,6 +394,22 @@ class FilesAPI(APIClient):
                 >>> c = CogniteClient()
                 >>> my_update = FileMetadataUpdate(id=1).source.set("new source").metadata.add({"key": "value"})
                 >>> res = c.files.update(my_update)
+
+            Attach labels to a files::
+
+                >>> from cognite.client import CogniteClient
+                >>> from cognite.client.data_classes import FileMetadataUpdate
+                >>> c = CogniteClient()
+                >>> my_update = FileMetadataUpdate(id=1).labels.add(["PUMP", "VERIFIED"])
+                >>> res = c.files.update(my_update)
+
+            Detach a single label from a file::
+
+                >>> from cognite.client import CogniteClient
+                >>> from cognite.client.data_classes import FileMetadataUpdate
+                >>> c = CogniteClient()
+                >>> my_update = FileMetadataUpdate(id=1).labels.remove("PUMP")
+                >>> res = c.files.update(my_update)
         """
         return self._update_multiple(cls=FileMetadataList, resource_path=self._RESOURCE_PATH, items=item)
 
@@ -395,6 +434,13 @@ class FilesAPI(APIClient):
                 >>> from cognite.client import CogniteClient
                 >>> c = CogniteClient()
                 >>> res = c.files.search(name="some name")
+
+            Search for an asset with an attached label:
+
+                >>> from cognite.client import CogniteClient
+                >>> c = CogniteClient()
+                >>> my_label_filter = LabelFilter(contains_all=["WELL LOG"])
+                >>> res = c.assets.search(name="xyz",filter=FileMetadataFilter(labels=my_label_filter))
         """
         return self._search(search={"name": name}, filter=filter, limit=limit)
 
@@ -402,6 +448,7 @@ class FilesAPI(APIClient):
         self,
         path: str,
         external_id: str = None,
+        directory: str = None,
         name: str = None,
         source: str = None,
         mime_type: str = None,
@@ -410,6 +457,7 @@ class FilesAPI(APIClient):
         source_created_time: int = None,
         source_modified_time: int = None,
         data_set_id: int = None,
+        labels: List[Label] = None,
         security_categories: List[int] = None,
         recursive: bool = False,
         overwrite: bool = False,
@@ -419,12 +467,14 @@ class FilesAPI(APIClient):
         Args:
             path (str): Path to the file you wish to upload. If path is a directory, this method will upload all files in that directory.
             external_id (str): The external ID provided by the client. Must be unique within the project.
+            directory (str): The directory provided by the client. Must be an absolute, unix-style path.
             name (str): Name of the file.
             source (str): The source of the file.
             mime_type (str): File type. E.g. text/plain, application/pdf, ...
             metadata (Dict[str, str]): Customizable extra data about the file. String key -> String value.
             asset_ids (List[int]): No description.
             data_set_id (int): ID of the data set.
+            labels (List[Label]): A list of the labels associated with this resource item.
             source_created_time (int): The timestamp for when the file was originally created in the source system.
             source_modified_time (int): The timestamp for when the file was last modified in the source system.
             recursive (bool): If path is a directory, upload all contained files recursively.
@@ -459,15 +509,24 @@ class FilesAPI(APIClient):
                 >>> c = CogniteClient()
                 >>> res = c.files.upload("/path/to/my/directory")
 
+            Upload a file with a label::
+
+                >>> from cognite.client import CogniteClient
+                >>> from cognite.client.data_classes import Label
+                >>> c = CogniteClient()
+                >>> res = c.files.upload("/path/to/file", name="my_file", labels=[Label(external_id="WELL LOG")])
+
         """
         file_metadata = FileMetadata(
             name=name,
+            directory=directory,
             external_id=external_id,
             source=source,
             mime_type=mime_type,
             metadata=metadata,
             asset_ids=asset_ids,
             data_set_id=data_set_id,
+            labels=labels,
             source_created_time=source_created_time,
             source_modified_time=source_modified_time,
             security_categories=security_categories,
@@ -509,12 +568,14 @@ class FilesAPI(APIClient):
         self,
         content: Union[str, bytes, TextIO, BinaryIO],
         name: str,
+        directory: str = None,
         external_id: str = None,
         source: str = None,
         mime_type: str = None,
         metadata: Dict[str, str] = None,
         asset_ids: List[int] = None,
         data_set_id: int = None,
+        labels: List[Label] = None,
         source_created_time: int = None,
         source_modified_time: int = None,
         security_categories: List[int] = None,
@@ -527,12 +588,14 @@ class FilesAPI(APIClient):
         Args:
             content (Union[str, bytes, TextIO, BinaryIO]): The content to upload.
             name (str): Name of the file.
+            directory (str): The directory provided by the client. Must be an absolute, unix-style path.
             external_id (str): The external ID provided by the client. Must be unique within the project.
             source (str): The source of the file.
             mime_type (str): File type. E.g. text/plain, application/pdf,...
             metadata (Dict[str, str]): Customizable extra data about the file. String key -> String value.
             asset_ids (List[int]): No description.
             data_set_id (int): Id of the data set.
+            labels (List[Label]): A list of the labels associated with this resource item.
             source_created_time (int): The timestamp for when the file was originally created in the source system.
             source_modified_time (int): The timestamp for when the file was last modified in the source system.
             overwrite (bool): If 'overwrite' is set to true, and the POST body content specifies a 'externalId' field,
@@ -554,12 +617,14 @@ class FilesAPI(APIClient):
         """
         file_metadata = FileMetadata(
             name=name,
+            directory=directory,
             external_id=external_id,
             source=source,
             mime_type=mime_type,
             metadata=metadata,
             asset_ids=asset_ids,
             data_set_id=data_set_id,
+            labels=labels,
             source_created_time=source_created_time,
             source_modified_time=source_modified_time,
             security_categories=security_categories,
@@ -571,7 +636,7 @@ class FilesAPI(APIClient):
         returned_file_metadata = res.json()
         upload_url = returned_file_metadata.pop("uploadUrl")
         headers = {"X-Upload-Content-Type": file_metadata.mime_type}
-        self._request_session.put(upload_url, data=content, headers=headers)
+        self._http_client.request("PUT", upload_url, data=content, timeout=180, headers=headers)
         return FileMetadata._load(returned_file_metadata)
 
     def download(
@@ -639,7 +704,7 @@ class FilesAPI(APIClient):
         self._download_file_to_path(download_link, file_path)
 
     def _download_file_to_path(self, download_link: str, path: str, chunk_size: int = 2 ** 21):
-        with self._request_session.get(download_link, stream=True) as r:
+        with self._http_client.request("GET", download_link, stream=True, timeout=180) as r:
             with open(path, "wb") as f:
                 for chunk in r.iter_content(chunk_size=chunk_size):
                     if chunk:  # filter out keep-alive new chunks
@@ -691,5 +756,5 @@ class FilesAPI(APIClient):
         return self._download_file(download_link)
 
     def _download_file(self, download_link: str) -> bytes:
-        res = self._request_session.get(download_link)
+        res = self._http_client.request("GET", download_link, timeout=180)
         return res.content
